<?php

namespace wisdmlabs\cpb;

if (! class_exists('WdmManagePrefillProducts')) {
    /**
    * This class is to handle the pre-filled products data.
    * Update, delete and insert the pre-filled products as required.
    * Gets the pre-filled products data from the database.
    */
    class WdmManagePrefillProducts
    {

        public $prefillProductTable;

        /**
         * @var Singleton The reference to *Singleton* instance of this class
         */
        private static $instance;
        public $errors;

        /**
         * Returns the *Singleton* instance of this class.
         *
         * @return Singleton The *Singleton* instance.
         */
        public static function getInstance()
        {
            if (null === static::$instance) {
                static::$instance = new static();
            }

            return static::$instance;
        }

        /**
         * Protected constructor to prevent creating a new instance of the
         * *Singleton* via the `new` operator from outside of this class.
         */
        protected function __construct()
        {
            global $wpdb;
            $this->prefillProductTable = $wpdb->prefix . 'cpb_prefilled_products_data';
        }

        /**
        * Attaches the error message field.
        * @param string $message message to be appended.
        */
        public function addError($message)
        {
            $this->errors .= $message;
        }

        /**
         * Retrives all pre-filled products information for all CPB products
         * @return array if CPB products contain pre-filled products then returns
         * an array of prefilled products information else empty array
         */
        public function getAllPrefilledProducts()
        {
            global $wpdb;

            $productsList = array();
            $prefillResult = $wpdb->get_results($wpdb->prepare("SELECT * FROM {$this->prefillProductTable}"));

            if ($prefillResult) {
                $key = 0;
                foreach ($prefillResult as $singleResult) {
                    $productsList[$key]['cpb_product_id'] = $singleResult->cpb_product_id;
                    $productsList[$key]['product_id'] = $singleResult->prefill_product_id;
                    $productsList[$key]['product_qty'] = $singleResult->prefill_quantity;
                    $productsList[$key]['product_type'] = $singleResult->product_type;
                    $productsList[$key]['product_mandatory'] = $singleResult->prefill_mandatory;
                    $key++;
                }
            }

            if (! empty($productsList)) {
                $productsList = array_filter($productsList);
            }

            return $productsList;
        }

        /**
         * Retrives information of the CPB product to which the pre-filled product
         * belongs
         * @param  int $productId pre-filled product id
         * @param  string $uniqueId pre-filled unique id
         * @return array if product id belongs to any CPB product then returns
         * array containing information about that CPB product else empty array
         */
        public function getCpbProducts($productId)
        {
            global $wpdb;

            $productsList = array();
            if (get_post_status($productId) == false) { //deleted product sync
                $wpdb->delete(
                    $this->prefillProductTable,
                    array(
                        'prefill_product_id'    => $productId,
                    ),
                    array(
                        '%s'
                    )
                );
            }

            $prefillResult = $wpdb->get_results($wpdb->prepare("SELECT cpb_product_id, prefill_quantity, prefill_mandatory, prefill_product_id, unique_prod_id, product_type FROM {$this->prefillProductTable} WHERE prefill_product_id = %s", $productId));

            if ($prefillResult) {
                $key = 0;
                foreach ($prefillResult as $singleResult) {
                    $productsList[$key]['unique_prod_id'] = $singleResult->unique_prod_id;
                    $productsList[$key]['product_id'] = $singleResult->prefill_product_id;
                    $productsList[$key]['cpb_product_id'] = $singleResult->cpb_product_id;
                    $productsList[$key]['product_qty'] = $singleResult->prefill_quantity;
                    $productsList[$key]['product_type'] = $singleResult->product_type;
                    $productsList[$key]['product_mandatory'] = $singleResult->prefill_mandatory;
                    $productsList[$key]['cpb_name'] = get_the_title($singleResult->cpb_product_id);
                    $key++;
                }
            }

            if (! empty($productsList)) {
                $productsList = array_filter($productsList);
            }

            return $productsList;
        }

        /**
         * Retrives information for all pre-filled products for a particular CPB
         * product
         * @param  int $cpbId CPB product id
         * @return array $productsList if CPB product contains pre-filled products
         * then returns an array of prefilled products information else empty array
         */
        public function getPrefilledProducts($cpbId)
        {
            global $wpdb;
            static $staticCPBId = 0;
            static $productsList = array();

            if ($staticCPBId == $cpbId) {
                return $productsList;
            }

            $staticCPBId = $cpbId;

            $prefillResult = $wpdb->get_results($wpdb->prepare("SELECT id, unique_prod_id, prefill_product_id, prefill_quantity, prefill_mandatory, product_type FROM {$this->prefillProductTable} WHERE cpb_product_id = %d", $cpbId));

            if ($prefillResult) {
                $key = 0;
                foreach ($prefillResult as $singleResult) {
                    if (get_post_status($singleResult->prefill_product_id) != false) {
                        $productsList[$key]['unique_prod_id'] = $singleResult->unique_prod_id;
                        $productsList[$key]['product_id'] = $singleResult->prefill_product_id;
                        $productsList[$key]['product_qty'] = $singleResult->prefill_quantity;
                        $productsList[$key]['product_mandatory'] = $singleResult->prefill_mandatory;
                        $productsList[$key]['product_type'] = $singleResult->product_type;
                        $key++;
                    } else {
                        $wpdb->delete($this->prefillProductTable, array(
                            'cpb_product_id'        => $cpbId,
                            'prefill_product_id'    => $singleResult->prefill_product_id,
                            'unique_prod_id'        => $singleResult->unique_prod_id,
                        ), array(
                            '%d',
                            '%d',
                            '%s',
                        ));
                    }
                }
            }

            if (! empty($productsList)) {
                $productsList = array_filter($productsList);
            }

            return $productsList;
        }

        /**
         * Retrives pre-filled product ids for a particular CPB product
         * @param  int $cpbId CPB product id
         * @return array $productsIds if CPB product contains pre-filled products
         * then returns an array containing ids of prefilled products else empty
         * array
         */
        public function getPrefilledProductIds($cpbId)
        {
            global $wpdb;

            $productsIds = array();
            $prefillResult = $wpdb->get_results($wpdb->prepare("SELECT unique_prod_id, prefill_product_id FROM {$this->prefillProductTable} WHERE cpb_product_id = %d", $cpbId));

            if ($prefillResult) {
                foreach ($prefillResult as $singleResult => $value) {
                    unset($singleResult);
                    if (get_post_status($value->prefill_product_id) != false) {
                        $productsIds[] = $value->unique_prod_id;
                    } else {
                        $wpdb->delete(
                            $this->prefillProductTable,
                            array(
                                'cpb_product_id'        => $cpbId,
                                'prefill_product_id'    => $value->prefill_product_id
                            ),
                            array(
                                '%d',
                                '%d',
                            )
                        );
                    }
                }
            }

            if (! empty($productsIds)) {
                $productsIds = array_filter($productsIds);
            }

            return $productsIds;
        }

        /**
         * Inserts pre-filled products data in DB
         * @param int $cpbId  CPB Product id
         * @param array $prefillProducts Array of all pre-filled products id
         * @param array $prefillQty  Array of all pre-filled products quantity
         * @param array $prefillMandatory Array of all mandatory pre-filled
         * products
         * @param  string $key position of value to be inserted
         * @param  string $value value to be inserted
         * @param  array  $insertValues Array of product ids to be inserted
         */
        public function insertPrefilledProducts($cpbId, $prefillProducts, $prefillQty, $prefillMandatory, $bundle_data, $key = "", $value = "", $insertValues = array())
        {
            global $wpdb;
            if (empty($insertValues)) {
                $prefillProducts = array_unique($prefillProducts);
                foreach ($prefillProducts as $index => $uniqueId) {
                    $prefillId = $bundle_data[$uniqueId]['product_type'] == "variation" ? $bundle_data[$uniqueId]['variation_id'] : $bundle_data[$uniqueId]['product_id'];
                    if (get_post_status($prefillId) != false) {
                        $wpdb->insert($this->prefillProductTable, array(
                            'cpb_product_id'         => $cpbId,
                            'unique_prod_id'         => $uniqueId,
                            'prefill_product_id'     => $prefillId,
                            'prefill_quantity'       => $prefillQty[$index],
                            'prefill_mandatory'      => $prefillMandatory[$index],
                            'product_type'       => $bundle_data[$uniqueId]['product_type'],
                        ), array(
                            '%d',
                            '%s',
                            '%d',
                            '%d',
                            '%d',
                            '%s'
                        ));
                    }
                }
            } else {
                    $prefillId = $bundle_data[$value]['product_type'] == "variation" ? $bundle_data[$value]['variation_id'] : $bundle_data[$value]['product_id'];
                if (get_post_status($prefillId) != false) {
                    $wpdb->insert($this->prefillProductTable, array(
                        'cpb_product_id'         => $cpbId,
                        'unique_prod_id'         => $value,
                        'prefill_product_id'     => $prefillId,
                        'prefill_quantity'       => $prefillQty[$key],
                        'prefill_mandatory'      => $prefillMandatory[$key],
                        'product_type'       => $bundle_data[$value]['product_type'],
                    ), array(
                        '%d',
                        '%s',
                        '%d',
                        '%d',
                        '%d',
                        '%s'
                    ));
                }
            }
        }


        /**
         * Updates pre-filled products data in DB
         * @param  int $cpbId  CPB Product id
         * @param Array $prefillProducts  Array of all pre-filled products id
         * @param Array $prefillQty  Array of all pre-filled products quantity
         * @param Array $prefillMandatory Array of all mandatory pre-filled products
         * @param  string $key position of value to be updated
         * @param  string $value  value to be updated
         * @param  array  $updateValues Array of product ids to be updated
         */
        public function updatePrefilledProducts($cpbId, $prefillProducts, $prefillQty, $prefillMandatory, $bundle_data, $key, $value, $updateValues)
        {
            global $wpdb;
            unset($prefillProducts);
            unset($updateValues);

            $prefillId = $bundle_data[$value]['product_type'] == "variation" ? $bundle_data[$value]['variation_id'] : $bundle_data[$value]['product_id'];
            if (get_post_status($prefillId) != false) {
                $wpdb->update($this->prefillProductTable, array(
                    'prefill_quantity'  => $prefillQty[$key],
                    'prefill_mandatory' => $prefillMandatory[$key],
                ), array(
                    'cpb_product_id'        => $cpbId,
                    'unique_prod_id'        => $value,
                    'prefill_product_id'    => $prefillId
                ), array(
                    '%d',
                    '%d',
                ), array(
                    '%d',
                    '%s',
                    '%d',
                ));
            } else { // If a product is deleted
                $wpdb->delete(
                    $this->prefillProductTable,
                    array(
                        'cpb_product_id'        => $cpbId,
                        'unique_prod_id'        => $value,
                    ),
                    array(
                        '%d',
                        '%s',
                    )
                );
            }
        }

        /**
         * Deletes pre-filled products from DB which are present in DB but not in
         * current selection.
         * Also , delete the pre-filled products details if pre-fill option not
         * checked
         * @param  int $cpbId  CPB Product id
         * @param  array  $deletedValues Array of product ids to be deleted which
         * are present in DB but not in current selection.
         */
        public function deletePrefilledProducts($cpbId, $deletedValues = array())
        {
            global $wpdb;

            if (empty($deletedValues)) {
                $existing = $wpdb->get_col($wpdb->prepare("SELECT id FROM {$this->prefillProductTable} WHERE cpb_product_id = %d", $cpbId));
                if (!empty($existing)) {
                    $existing = array_values($existing);
                    $deleteCount = count($existing);

                    if ($deleteCount > 0) {
                        $deletePlaceholders = array_fill(0, $deleteCount, '%d');

                        $placeholders = implode(',', $deletePlaceholders);

                        $deleteQuery = "DELETE FROM {$this->prefillProductTable} WHERE id IN ($placeholders)";
                        $wpdb->query($wpdb->prepare($deleteQuery, $existing));
                    }
                }
            } else {
                $existing = $wpdb->get_col($wpdb->prepare("SELECT id FROM {$this->prefillProductTable} WHERE cpb_product_id = %d and unique_prod_id IN ('" . implode("', '", $deletedValues) . "')", $cpbId));
                if (!empty($existing)) {
                    $existing = array_values($existing);
                    $deleteCount = count($existing);

                    if ($deleteCount > 0) {
                        $deletePlaceholders = array_fill(0, $deleteCount, '%d');

                        $placeholders = implode(',', $deletePlaceholders);

                        $deleteQuery = "DELETE FROM {$this->prefillProductTable} WHERE id IN ($placeholders)";

                        $wpdb->query($wpdb->prepare($deleteQuery, $existing));
                    }
                }
            }
        }

        /**
         * Processes records for insert, update or delete
         * Get the saved pre-filled product ids from the database.
         * If there are some existing values for pre-filled products.
         * If the existing values are not present in the current selection delete
         * them from the database.
         * Update the current selected pre-filled products in the DB.
         * @param  int $cpbId  CPB Product id
         * @param  Array $prefillProducts Array of all pre-filled products ids
         * @param  Array $prefillQty Array of all pre-filled products quantity
         * @param  Arra] $prefillMandatory Array of all mandatory pre-filled
         * products
         */
        public function savePrefilledProducts($cpbId, $prefillProducts, $prefillQty, $prefillMandatory, $bundle_data)
        {
            // global $wpdb;
            $insertValues = array();
            $deletedValues = array();
            $updateValues = array();
            //Get the saved pre-filled product ids form database.
            $existing = $this->getPrefilledProductIds($cpbId);

            if (!empty($existing)) {
                $insertValues = array_diff($prefillProducts, $existing);
                $deletedValues = array_diff($existing, $prefillProducts);
                $updateValues = array_intersect($existing, $prefillProducts);

                if (!empty($deletedValues)) {
                    $this->deletePrefilledProducts($cpbId, $deletedValues);
                }
                foreach ($prefillProducts as $key => $value) {
                    if (!empty($insertValues) && in_array($value, $insertValues)) {
                        $this->insertPrefilledProducts($cpbId, $prefillProducts, $prefillQty, $prefillMandatory, $bundle_data, $key, $value, $insertValues);
                    }
                    if (!empty($updateValues) && in_array($value, $updateValues)) {
                        $this->updatePrefilledProducts($cpbId, $prefillProducts, $prefillQty, $prefillMandatory, $bundle_data, $key, $value, $updateValues);
                    }
                } // end foreach
            } else {
                $this->insertPrefilledProducts($cpbId, $prefillProducts, $prefillQty, $prefillMandatory, $bundle_data);
            }
        }


        // public function updatePrefilled($prefilledData, $cpbId)
        // {
        //     foreach ($prefilledData as $key => $value) {
        //         if (get_post_status($value['product_id']) != false) {
        //             $wpdb->update($this->prefillProductTable, array(
        //                 'prefill_quantity'  => $value['product_qty'],
        //                 'prefill_mandatory' => $value['product_mandatory'],
        //             ), array(
        //                 'cpb_product_id'        => $cpbId,
        //                 'unique_prod_id'        => $value['unique_prod_id'],
        //                 'prefill_product_id'    => $value['product_id']
        //             ), array(
        //                 '%d',
        //                 '%d',
        //             ), array(
        //                 '%d',
        //                 '%s',
        //                 '%d',
        //             ));
        //         } else { // If a product is deleted
        //             $wpdb->delete(
        //                 $this->prefillProductTable,
        //                 array(
        //                     'cpb_product_id'        => $cpbId,
        //                     'unique_prod_id'        => $value['unique_prod_id'],
        //                 ),
        //                 array(
        //                     '%d',
        //                     '%s',
        //                 )
        //             );
        //         }
        //     }
        // }
    }
}
$GLOBALS['prefillManager'] = WdmManagePrefillProducts::getInstance();
